﻿using Decal.Adapter;
using Decal.Adapter.Wrappers;
using System;
using System.Collections.Generic;
using System.Text.RegularExpressions;

namespace CompHud
{
    public class ItemMatcher
    {
        private Regex name;
        private int mat = -1;
        private int work = -1;

        public ItemMatcher(String regex)
        {
            name = new Regex(regex == null || String.IsNullOrEmpty(regex.Trim()) ? ".*" : regex, RegexOptions.IgnoreCase);    
        }

        public ItemMatcher(int material)
        {
            mat = material;
        }

        public ItemMatcher(int material, int workmanship) : this(material)
        {
            work = workmanship;
        }

        public ItemMatcher(String regex, int material, int workmanship) : this(regex)
        {
            mat = material;
            work = workmanship;
        }

        public bool IsMatch(WorldObject wo)
        {
            if (name == null || name.IsMatch(wo.Name))
            {
                if (mat < 1 || mat == wo.Values(LongValueKey.Material, 0))
                {
                    return work < 1 || work == Math.Floor(wo.Values(DoubleValueKey.SalvageWorkmanship, 0));
                }
            }

            return false;
        }

        public override string ToString() {
            List<String> parts = new List<string>();
            if (name != null)
            {
                parts.Add(string.Format("name={0}", name));
            }
            if (mat > 0)
            {
                parts.Add(string.Format("material={0}", PluginCore.getMaterialName(mat)));
            }
            if (work > 0)
            {
                parts.Add(string.Format("work={0}", work));
            }
            return string.Join(" ", parts.ToArray());
        }
    }
}
