/* Plugin template by Timo 'lino' Kissing <http://ac.ranta.info/DecalDev> */
/* Original template by Lonewolf <http://www.the-lonewolf.com> */

using Decal.Adapter;
using Decal.Filters;
using System;
using System.IO;
using System.Collections.Generic;
using VirindiViewService;

namespace CompHud
{
    public partial class PluginCore : PluginBase
    {

        private static string DIR_SEP = "\\";
        private static string PLUGIN = "CompHud";

        private static string FILENAME_ERRORLOG = "errorlog.txt";
        private string FILENAME_SETTINGS {
            get { return Core.CharacterFilter.Id != 0 ? string.Format("{0}.xml", Core.CharacterFilter.Name) : "settings.xml"; }
        }

        private bool inactive = false;

        private List<DelayedChatParserText> timedChatParserCommands = new List<DelayedChatParserText>();

        public string settingsBaseFolder
        {
            get
            {
                return Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments) + DIR_SEP + "Decal Plugins" + DIR_SEP + PLUGIN;
            }
        }

        public string settingsFolder { get { return settingsBaseFolder + DIR_SEP + Core.CharacterFilter.Server; } }

        public string settingsFile { get { return settingsFolder + DIR_SEP + FILENAME_SETTINGS; } }

        public string templateFolder { get { return settingsBaseFolder + DIR_SEP + "_templates"; } }

        public string errorLogFile { get { return settingsFolder + DIR_SEP + FILENAME_ERRORLOG; } }

        private SpellTable spellTable;

        protected override void Startup()
        {
            try
            {
                initCharStats();
                initWorldFilter();
                initChatEvents();

                initPath();

                spellTable = Core.Filter<FileService>().SpellTable;

                inactive = !VirindiViewsPresent(Host);

                //TODO: Code for startup events
            }
            catch (Exception ex)
            {
                ErrorLogging.LogError(errorLogFile, ex);
            }
        }

        protected override void Shutdown()
        {
            try
            {
                inactive = true;

                destroyChatEvents();
                destroyCharStats();
                destroyWorldFilter();

                stopTimer();
                stopMoveTimer();

                if (head != null) head.Dispose();
                if (hud != null) hud.Dispose();

                if (cHead != null) cHead.Dispose();
                if (cHud != null) cHud.Dispose();

                //TODO: Code for shutdown events
            }
            catch (Exception ex)
            {
                ErrorLogging.LogError(errorLogFile, ex);
            }
        }

        protected void initPath()
        {
            if (!Directory.Exists(settingsFolder))
            {
                try
                {
                    Directory.CreateDirectory(settingsFolder);
                }
                catch (Exception ex)
                {
                    ErrorLogging.LogError("c:\\CompHud-errorlog.txt", ex);
                }
            }
            if (!Directory.Exists(templateFolder))
            {
                try
                {
                    Directory.CreateDirectory(settingsFolder);
                }
                catch (Exception ex)
                {
                    ErrorLogging.LogError("c:\\CompHud-errorlog.txt", ex);
                }
            }
        }

        protected void moveLegacySettings()
        {
            if (File.Exists(settingsBaseFolder + DIR_SEP + FILENAME_SETTINGS))
            {
                try
                {
                    if (File.Exists(settingsFile))
                    {
                        File.Delete(settingsBaseFolder + DIR_SEP + FILENAME_SETTINGS);
                    }
                    else
                    {
                        File.Move(settingsBaseFolder + DIR_SEP + FILENAME_SETTINGS, settingsFile);
                    }
                }
                catch (Exception ex)
                {
                    ErrorLogging.LogError("c:\\CompHud-errorlog.txt", ex);
                }
            }
        }

        static bool VirindiViewsPresent(Decal.Adapter.Wrappers.PluginHost pHost)
        {
            Guid g = new Guid("DBAC9286-B38D-4570-961F-D4D9349AE3D4");

            Decal.Interop.Core.DecalEnum esv = null;
            try
            {
                esv = pHost.Underlying.Decal.get_Configuration("Services", ref g);
                return esv.Enabled;
            }
            catch { }
            finally
            {
                if (esv != null)
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(esv);
            }
            return false;
        }
    }
}