/* Plugin created by lino 2010 */

using System;
using System.IO;
using System.Xml.Serialization;
using System.Collections.Generic;

namespace CompHud
{
    public partial class PluginCore
    {
        public PluginSettings pluginSettings;

        private void loadSettings()
        {
            pluginSettings = PluginSettings.load(settingsFile, errorLogFile);
            initSettings();
        }

        private void saveSettings()
        {
            pluginSettings.save(settingsFile, errorLogFile);
            initSettings();
        }

        private void saveSettingsTemplate(String basename)
        {
            pluginSettings.save(templateFolder + DIR_SEP + basename + ".template.xml", errorLogFile);                     
        }

        private void loadSettingsTemplate(String basename)
        {
            pluginSettings = PluginSettings.load(templateFolder + DIR_SEP + basename + ".template.xml", errorLogFile);
            initSettings();
        }

        private void initSettings()
        {
            drawHud();
        }
    }

    public class PluginSettings
    {
        /* ADD PUBLIC PROPERTIES FOR PERSISTABLE SETTINGS HERE */
        public SerializableDictionary<string, int> WatchedItems;
        public bool visible;
        public bool textOff;
        public int alpha;
        public int chatTarget = 2;

        public PluginSettings()
        {
            WatchedItems = new SerializableDictionary<string, int>();
            alpha = 10;
            visible = false;
            textOff = false;
            chatTarget = 2;
            /* SET DEFAULTS FOR THE PROPERTIES HERE */
        }

        public static PluginSettings load(string file, string errorLogFile)
        {
            try
            {
                if (File.Exists(file))
                {
                    using (FileStream myFileStream = new FileStream(file, FileMode.Open))
                    {
                        XmlSerializer mySerializer = new XmlSerializer(typeof(PluginSettings));
                        PluginSettings mySettings = (PluginSettings)mySerializer.Deserialize(myFileStream);
                        myFileStream.Close();
                        return mySettings;
                    }
                }
            }
            catch (Exception ex)
            {
                ErrorLogging.LogError(errorLogFile, ex);
            }
            return new PluginSettings();
        }

        public void save(string file, string errorLogFile)
        {
            try
            {
                using (StreamWriter myWriter = new StreamWriter(file))
                {
                    XmlSerializer mySerializer = new XmlSerializer(typeof(PluginSettings));
                    myWriter.AutoFlush = true;
                    mySerializer.Serialize(myWriter, this);
                    myWriter.Close();
                }
            }
            catch (Exception ex)
            {
                ErrorLogging.LogError(errorLogFile, ex);
            }
        }
    }
}