﻿using Decal.Adapter.Wrappers;
using System;
using System.Collections.Generic;

namespace CompHud
{
    class WorldObjectInfo
    {
        static int LongValueKeySetId = 0x109;

        private static List<string> deathItemNames;

        public static bool isDeathItem(string Name)
        {
            if (deathItemNames == null)
            {
                deathItemNames = new List<string>();
                deathItemNames.Add("Titan Mana Charge");
                deathItemNames.Add("Massive Mana Charge");
                deathItemNames.Add("Dho Creature Master Robe");
                deathItemNames.Add("Dho Item Master Robe");
                deathItemNames.Add("Dho Life Master Robe");
                deathItemNames.Add("Dho War Master Robe");
                deathItemNames.Add("Faran Creature Master Robe");
                deathItemNames.Add("Faran Item Master Robe");
                deathItemNames.Add("Faran Life Master Robe");
                deathItemNames.Add("Faran War Master Robe");
                deathItemNames.Add("Suikan Creature Master Robe");
                deathItemNames.Add("Suikan Item Master Robe");
                deathItemNames.Add("Suikan Life Master Robe");
                deathItemNames.Add("Suikan War Master Robe");
                deathItemNames.Add("A Horn of Vigilance");
            }

            return deathItemNames.Contains(Name);
        }

        public static bool isRare(WorldObject wo)
        {
            return isRare(wo.Values(LongValueKey.IconUnderlay, 0));
        }

        public static bool isRare(int iconUnderlay)
        {
            return iconUnderlay == 23308;
        }

        public static int getItemsSetId(WorldObject wo)
        {
            return wo.Values((LongValueKey)LongValueKeySetId, 0);
        }

        public static int getItemsMaxLevel(WorldObject wo)
        {
            return getItemsMaxLevel(wo.Values(LongValueKey.IconOverlay, 0));
        }

        public static int getItemsMaxLevel(int iconOverlay)
        {
            switch (iconOverlay)
            {
                case 27700: return 1;
                case 27701: return 2;
                case 27702: return 3;
                case 27703: return 4;
                case 27704: return 5;
                case 27705: return 6;
                case 27706: return 7;
                case 27707: return 8;
                case 27708: return 9;
            }

            return 0;
        }

        public static string getElementalType(int elementalDamageType, int damageType)
        {
            switch (elementalDamageType)
            {
                case 1:
                    return "Slash";
                case 2:
                    return "Pierc";
                case 4:
                    return "Bludg";
                case 8:
                    return "Cold";
                case 16:
                    return "Fire";
                case 32:
                    return "Acid";
                case 64:
                    return "Light";
                default:
                    switch (damageType)
                    {
                        case 1:
                            return "Slash";
                        case 2:
                            return "Pierc";
                        case 4:
                            return "Bludg";
                        case 8:
                            return "Cold";
                        case 16:
                            return "Fire";
                        case 32:
                            return "Acid";
                        case 64:
                            return "Light";
                        default:
                            return String.Format("DamageType{0}#{1}", elementalDamageType, damageType);
                    }
            }
        }

        public static string getHeritageName(int heritage)
        {
            switch (heritage)
            {
                case 1:
                    return "Aluv";
                case 2:
                    return "Gharu";
                case 3:
                    return "Sho";
                case 4:
                    return "Viamontian";
                case 9:
                    return "Empyrean";
                case 10:
                    return "Penumbraen";
                default:
                    return string.Empty;
            }
        }

        public static string getImbueName(int imbue)
        {
            List<string> imbues = new List<string>();
            if ((imbue & 1) == 1)
            {
                imbues.Add("CS");
            }
            if ((imbue & 2) == 2)
            {
                imbues.Add("CB");
            }
            if ((imbue & 4) == 4)
            {
                imbues.Add("AR");
            }
            if ((imbue & 8) == 8)
            {
                imbues.Add("SlashR");
            }
            if ((imbue & 16) == 16)
            {
                imbues.Add("PiercR");
            }
            if ((imbue & 32) == 32)
            {
                imbues.Add("BludgR");
            }
            if ((imbue & 64) == 64)
            {
                imbues.Add("AcidR");
            }
            if ((imbue & 128) == 128)
            {
                imbues.Add("FrostR");
            }
            if ((imbue & 256) == 256)
            {
                imbues.Add("LightR");
            }
            if ((imbue & 512) == 512)
            {
                imbues.Add("FireR");
            }
            if ((imbue & 1024) == 1024)
            {
                imbues.Add("MeleeImbue");
            }
            if ((imbue & 8192) == 8192)
            {
                imbues.Add("Imbued");
            }
            if ((imbue & 536870912) == 536870912)
            {
                imbues.Add("MagicAbsorbing");
            }
            return string.Join(",", imbues.ToArray()).Trim(new char[] { ',' });
        }

        public static string getSetName(int setId)
        {
            switch (setId)
            {
                case 88:
                    return "(W) Sneak Attack";
                case 87:
                    return "(W) Shield";
                case 86:
                    return "(W) Recklessness";
                case 85:
                    return "(W) Dual Wield";
                case 84:
                    return "(W) Dirty Fighting";
                case 83:
                    return "(W) Assess Creature";
                case 82:
                    return "(W) Weapon Tinkering";
                case 81:
                    return "(W) War Magic";
                case 80:
                    return "(W) Void Magic";
                case 78:
                    return "(W) Two Handed Weapons";
                case 76:
                    return "(W) Heavy Weapons";
                case 73:
                    return "(W) Salvaging";
                case 72:
                    return "(W) Missile Defense";
                case 71:
                    return "(W) Melee Defense";
                case 70:
                    return "(W) Mana Conversion";
                case 69:
                    return "(W) Magic Item Tinkering";
                case 68:
                    return "(W) Magic Defense";
                case 66:
                    return "(W) Loyalty";
                case 65:
                    return "(W) Life Magic";
                case 64:
                    return "(W) Leadership";
                case 63:
                    return "(W) Item Tinkering";
                case 62:
                    return "(W) Item Enchantment";
                case 61:
                    return "(W) Healing";
                case 60:
                    return "(W) Fletching";
                case 59:
                    return "(W) Deception";
                case 58:
                    return "(W) Finesse Weapons";
                case 56:
                    return "(W) Creature Enchantment";
                case 55:
                    return "(W) Cooking";
                case 54:
                    return "(W) Missile Weapons";
                case 53:
                    return "(W) Light Weapons";
                case 52:
                    return "(W) Assess Person";
                case 51:
                    return "(W) Armor Tinkering";
                case 50:
                    return "(W) Arcane Lore";
                case 49:
                    return "(W) Alchemy";
                case 41:
                    return "Heroic Destroyer";
                case 40:
                    return "Heroic Defender";
                case 39:
                    return "Vigor";
                case 38:
                    return "Growth";
                case 37:
                    return "Fury";
                case 36:
                    return "Destruction";
                case 35:
                    return "Defense";
                case 32:
                    return "Protective Cloth";
                case 31:
                    return "Gladiatorial Cloth";
                case 30:
                    return "Dedication";
                case 29:
                    return "Light Proof";
                case 28:
                    return "Cold Proof";
                case 27:
                    return "Acid Proof";
                case 26:
                    return "Flame Proof";
                case 25:
                    return "Interlocking";
                case 24:
                    return "Reinforced";
                case 23:
                    return "Hardenend";
                case 22:
                    return "Rogue/Swift";
                case 21:
                    return "Wise";
                case 20:
                    return "Dexterous";
                case 19:
                    return "Hearty";
                case 18:
                    return "Crafter's";
                case 17:
                    return "Tinker's";
                case 16:
                    return "Defender's";
                case 15:
                    return "Archer's";
                case 14:
                    return "Adept's";
                case 13:
                    return "Soldier's";
                case 12:
                    return "Leggings of Perfect Light";
                case 11:
                    return "Coat of the Perfect Light";
                case 10:
                    return "Arm Mind Heart";
                case 9:
                    return "Empyrean Rings";
                case 8:
                    return "Shou-jen";
                case 7:
                    return "Relic Alduressa";
                case 6:
                    return "Ancient Relic";
                case 5:
                    return "Noble Relic";
                default:
                    return String.Format("SetId{0}", setId);
            }
        }

        public static string getSkillName(int skillId)
        {
            switch (skillId)
            {
                case 1:
                    return "Axe";
                case 2:
                    return "Bow";
                case 3:
                    return "XBow";
                case 4:
                    return "Dagger";
                case 5:
                    return "Mace";
                case 6:
                    return "MeleeD";
                case 7:
                    return "MissileD";
                case 9:
                    return "Spear";
                case 10:
                    return "Staff";
                case 11:
                    return "Sword";
                case 12:
                    return "TW";
                case 13:
                    return "UA";
                case 15:
                    return "MagicD";
                case 16:
                    return "ManaCon";
                case 31:
                    return "Creature";
                case 32:
                    return "Item";
                case 33:
                    return "Life";
                case 34:
                    return "War";
                case 41:
                    return "2H";
                case 43:
                    return "Void";
                case 44:
                    return "HW";
                case 45:
                    return "LW";
                case 46:
                    return "FW";
                case 47:
                    return "MissileW";
                case 48:
                    return "Shield";
                case 49:
                    return "DW";
                case 50:
                    return "Reckless";
                case 51:
                    return "Sneak";
                case 52:
                    return "Dirty";
                default:
                    return String.Format("SkillId{0}", skillId);
            }
        }

        public static string getSpeciesName(int species)
        {
            switch (species)
            {
                case 1:
                    return "Olthoi";
                case 8:
                    return "Tusker";
                case 14:
                    return "Undead";
                case 19:
                    return "Virindi";
                case 22:
                    return "Shadow";
                case 31:
                    return "Human";
                case 62:
                    return "Elemental";
                case 77:
                    return "Ghost";
                case 83:
                    return "ViamontKnight";
                case 89:
                    return "Mukkir";
                default:
                    return String.Format("Species{0}", species);
            }
        }

    }
}
