﻿using Decal.Adapter;
using Decal.Adapter.Wrappers;
using System;
using System.IO;
using System.Text.RegularExpressions;

namespace CompHud
{
    public partial class PluginCore : PluginBase
    {
        int getItemCount(string name)
        {
            return getItemCount(name, false);
        }

        int getItemCount(string name, bool includePartialMatches)
        {
            return getItemCount(name, includePartialMatches, 0);
        }

        int getItemCount(string name, bool includePartialMatches, int container)
        {
            int found = 0;
            foreach (WorldObject wo in Core.WorldFilter.GetInventory())
            {
                if (nameKindaMatches(name, wo.Name) || (includePartialMatches && wo.Name.ToLower().Contains(name.ToLower())))
                {
                    if (container == 0 || wo.Container == container) found += wo.Values(LongValueKey.StackCount, 1);
                }
            }
            return found;
        }

        bool nameKindaMatches(string search, string input)
        {
            return search.Equals(input) || (Array.IndexOf(MANA_CHARGES, search) > -1 && Array.IndexOf(MANA_CHARGES, input) > -1);
        }

        bool isSalvage(WorldObject wo)
        {
            return wo != null && wo.ObjectClass == ObjectClass.Salvage;
        }

        bool isUnderwear(WorldObject wo)
        {
            return wo != null && wo.ObjectClass == ObjectClass.Clothing && (wo.Values(LongValueKey.EquipableSlots, 0) & 222) != 0;
        }

        bool isArmor(WorldObject wo)
        {
            return wo != null && (wo.ObjectClass == ObjectClass.Armor || (wo.ObjectClass == ObjectClass.Clothing && !isUnderwear(wo) && !isCloak(wo)));
        }

        bool isCloak(WorldObject wo)
        {
            return wo != null && wo.ObjectClass == ObjectClass.Clothing && (wo.Values(LongValueKey.EquipableSlots, 0) & 134217728) != 0;
        }

        bool isAetheria(WorldObject wo)
        {
            return wo != null && wo.ObjectClass == ObjectClass.Gem && "Aetheria".Equals(wo.Name);
        }

        bool isJewelry(WorldObject wo)
        {
            return wo != null && wo.ObjectClass == ObjectClass.Jewelry;
        }

        bool isEquipableItem(WorldObject wo)
        {
            return wo != null && wo.Values(LongValueKey.EquipableSlots, 0) > 0;
        }

        bool isWand(WorldObject wo)
        {
            return wo != null && wo.ObjectClass == ObjectClass.WandStaffOrb;
        }

        bool isWeapon(WorldObject wo)
        {
            return wo != null && (wo.ObjectClass == ObjectClass.MeleeWeapon || wo.ObjectClass == ObjectClass.MissileWeapon);
        }

        bool isWandOrWeapon(WorldObject wo)
        {
            return isWand(wo) || isWeapon(wo);
        }

        bool isInPack(WorldObject wo, int packGuid, bool strict)
        {
            return wo != null && wo.Container == packGuid && (!strict || wo.Values(LongValueKey.EquippedSlots, 0) == 0);
        }

        int mainPackFreeSlots()
        {
            int slotsFree = 102;
            foreach (WorldObject wo in Core.WorldFilter.GetInventory())
            {
                bool isInMain = wo.ObjectClass != ObjectClass.Container
                    && wo.ObjectClass != ObjectClass.Foci
                    && wo.Container == Core.CharacterFilter.Id
                    && wo.Values(LongValueKey.EquippedSlots, 0) == 0;
                if (isInMain)
                {
                    slotsFree--;
                }
            }
            return slotsFree;
        }

        int packFreeSlots(int pack)
        {
            if (pack == Core.CharacterFilter.Id) return mainPackFreeSlots();
            try
            {
                return Core.WorldFilter[pack].Values(LongValueKey.ItemSlots, 0) - Core.WorldFilter.GetByContainer(pack).Count;
            }
            catch (Exception ex)
            {

            }
            return 0;
        }

        int getSingleValue(WorldObject wo)
        {
            return wo == null ? 0 : wo.Values(LongValueKey.Value, 0) / wo.Values(LongValueKey.StackCount, 1);
        }

        string getEffectiveObjectClass(WorldObject wo)
        {
            if (isUnderwear(wo)) return "Underwear";
            if (isCloak(wo)) return "Cloak";
            if (isArmor(wo)) return "Armor";
            if (isAetheria(wo)) return "Aetheria";
            return string.Format("{0}", wo.ObjectClass);
        }

        string getSpellNameForSorting(WorldObject wo)
        {
            for (int i = 0; i < wo.SpellCount; i++)
            {
                int sid = wo.Spell(i);
                string sn = spellTable.GetById(sid).Name;
                if (sn.StartsWith("Epic ")) return sn;
            }
            for (int i = 0; i < wo.SpellCount; i++)
            {
                int sid = wo.Spell(i);
                string sn = spellTable.GetById(sid).Name;
                //WriteToChat(sn);
                if (sn.StartsWith("Major ")) return sn;
            }
            for (int i = 0; i < wo.SpellCount; i++)
            {
                int sid = wo.Spell(i);
                string sn = spellTable.GetById(sid).Name;
                //WriteToChat(sn);
                if (sn.StartsWith("Incantation of ")) return "With Level 8: " + sn;
            }
            for (int i = 0; i < wo.SpellCount; i++)
            {
                int sid = wo.Spell(i);
                string sn = spellTable.GetById(sid).Name;
                //WriteToChat(sn);
                if (sn.StartsWith("Moderate ")) return "With plus 10: " + sn;
            }
            for (int i = 0; i < wo.SpellCount; i++)
            {
                int sid = wo.Spell(i);
                string sn = spellTable.GetById(sid).Name;
                //WriteToChat(sn);
                if (sn.StartsWith("Minor ")) return "With plus 05: " + sn;
            }

            return "ZZZ (No cantrip or level 8 found)";
        }
    }
}