using Decal.Adapter;
using Decal.Adapter.Wrappers;
using System;
using System.Drawing;
using System.Text.RegularExpressions;
using VirindiViewService;
using VirindiViewService.Controls;

namespace CompHud
{
    public partial class PluginCore
    {
        private int MessageColor = 5;

        // currently have to type a number for icon id even when removing; TODO: improve regex
        private Regex AddDelRegex = new Regex("/ch (?<act>[+-]) (?<what>[^0-9]+) (?<icon>[0-9-]+)", RegexOptions.Compiled);
        private Regex AlphaRegex = new Regex("/ch alpha (?<value>[0-9]+)");
        private Regex SortRegex = new Regex("/sort (?<value>[0-9])");
        private Regex CramSalvageRegex = new Regex("/crams (?<value>[1-9]) (?<material>[a-zA-Z ]+) ?(?<work>[0-9]{1,2})?");
        private Regex CramRegex = new Regex("/cram (?<value>[1-9]) (?<name>[a-zA-Z0-9 .*]+)");
        private Regex EmptyRegex = new Regex("/empty (?<value>[1-9]) ?(?<name>[a-zA-Z0-9 .*]+)?");

        private static String CMD_SAVE = "/ch save ";
        private static String CMD_LOAD = "/ch load ";

        private void initChatEvents()
        {
            // Initialize incoming chat message event handler
            // Core.ChatBoxMessage += new EventHandler<Decal.Adapter.ChatTextInterceptEventArgs>(Core_ChatBoxMessage);

            // Initialize the outgoing chat/command message event handler
            Core.CommandLineText += new EventHandler<Decal.Adapter.ChatParserInterceptEventArgs>(Core_CommandLineText);
        }

        void Core_CommandLineText(object sender, Decal.Adapter.ChatParserInterceptEventArgs e)
        {
            try
            {
                if (pluginSettings != null && !e.Eat && e.Text.StartsWith("/ch "))
                {
                    e.Eat = true;

                    string what = string.Empty;
                    int icon = 0;
                    bool add = false;
                    bool del = false;

                    if (e.Text.Equals("/ch on"))
                    {
                        pluginSettings.visible = true;
                    }
                    else if (e.Text.Equals("/ch off"))
                    {
                        pluginSettings.visible = false;
                    }
                    else if (e.Text.Equals("/ch clear"))
                    {
                        pluginSettings.WatchedItems.Clear();
                    }
                    else if (e.Text.Equals("/ch stop"))
                    {
                        stopAll();
                    }
                    else if (e.Text.StartsWith("/ch text"))
                    {
                        pluginSettings.textOff = e.Text.Equals("/ch text off");
                    }
                    else if (e.Text.Equals("/ch +"))
                    {
                        if (Host.Actions.CurrentSelection != 0)
                        {
                            WorldObject wo = Core.WorldFilter[Host.Actions.CurrentSelection];
                            if (wo != null)
                            {
                                what = wo.Name;
                                icon = wo.Icon;
                                add = true;
                            }
                        }
                    }
                    else if (e.Text.Equals("/ch -"))
                    {
                        if (Host.Actions.CurrentSelection != 0)
                        {
                            WorldObject wo = Core.WorldFilter[Host.Actions.CurrentSelection];
                            if (wo != null)
                            {
                                what = wo.Name;
                                del = true;
                            }
                        }
                    }
                    else if (AlphaRegex.IsMatch(e.Text))
                    {
                        int a = Convert.ToInt32(AlphaRegex.Match(e.Text).Result("${value}").Trim());
                        pluginSettings.alpha = Math.Max(0, Math.Min(255, a));
                    }
                    else if (AddDelRegex.IsMatch(e.Text))
                    {
                        add = AddDelRegex.Match(e.Text).Result("${act}").Trim().Equals("+");
                        what = AddDelRegex.Match(e.Text).Result("${what}").Trim();
                        if (what.Length > 0)
                        {
                            if (add)
                            {
                                string i = AddDelRegex.Match(e.Text).Result("${icon}").Trim();
                                icon = i.Length > 0 ? Convert.ToInt32(i) : DEFAULTICON;
                                add = true;
                            }
                            else
                            {
                                del = true;
                            }
                        }
                    }
                    else if (e.Text.StartsWith(CMD_SAVE))
                    {
                        saveSettingsTemplate(e.Text.Replace(CMD_SAVE, String.Empty));
                    }
                    else if (e.Text.StartsWith(CMD_LOAD))
                    {
                        loadSettingsTemplate(e.Text.Replace(CMD_LOAD, String.Empty));
                    }
                    if (del && pluginSettings.WatchedItems.ContainsKey(what))
                    {
                        pluginSettings.WatchedItems.Remove(what);
                        WriteToChat(String.Format("Removed {0}", what));
                    }
                    else if (add && !pluginSettings.WatchedItems.ContainsKey(what))
                    {
                        pluginSettings.WatchedItems.Add(what, icon);
                        WriteToChat(String.Format("Added {0} with icon {1}", what, icon));
                    }
                    saveSettings();

                }
                else if (e.Text.StartsWith("/sort tinks "))
                {
                    int a = Convert.ToInt32(SortRegex.Match(e.Text.Replace(" tinks ", " ")).Result("${value}"));
                    sortContainer(a, tinkSortKey);
                    e.Eat = true;
                }
                else if (e.Text.StartsWith("/sort value "))
                {
                    int a = Convert.ToInt32(SortRegex.Match(e.Text.Replace(" value ", " ")).Result("${value}"));
                    sortContainer(a, valueSortKey);
                    e.Eat = true;
                }
                else if (e.Text.StartsWith("/sort set "))
                {
                    int a = Convert.ToInt32(SortRegex.Match(e.Text.Replace(" set ", " ")).Result("${value}"));
                    sortContainer(a, setSortKey);
                    e.Eat = true;
                }
                else if (e.Text.StartsWith("/sort "))
                {
                    int a = Convert.ToInt32(SortRegex.Match(e.Text).Result("${value}"));
                    sortContainer(a, defaultSortKey);
                    e.Eat = true;
                }
                else if (CramSalvageRegex.IsMatch(e.Text))
                {
                    Match m = CramSalvageRegex.Match(e.Text);
                    int material = GetMaterialID(m.Result("${material}").Trim());
                    string work = m.Result("${work}");
                    ItemMatcher matcher = new ItemMatcher(material, String.IsNullOrEmpty(work) ? 0 : Convert.ToInt32(work));
                    tryCramToContainer(m.Result("${value}"), matcher);
                    e.Eat = true;
                }
                else if (CramRegex.IsMatch(e.Text))
                {
                    Match m = CramRegex.Match(e.Text);
                    tryCramToContainer(m.Result("${value}"), new ItemMatcher(m.Result("${name}")));
                    e.Eat = true;
                }
                else if (EmptyRegex.IsMatch(e.Text))
                {
                    Match m = EmptyRegex.Match(e.Text);
                    tryUnCramFromContainer(m.Result("${value}"), new ItemMatcher(m.Result("${name}")));
                    e.Eat = true;
                }

                else if (e.Text.StartsWith("/count"))
                {
                    string what = e.Text.Substring(6).Trim();
                    if (what.Length < 1)
                    {
                        WorldObject wo = Core.WorldFilter[Host.Actions.CurrentSelection];
                        if (wo != null) what = wo.Name;
                    }
                    if (what.Length > 0)
                    {
                        WriteToChat(string.Format("You have {0} items with a name containing {1}.", getItemCount(what, true), what));
                        e.Eat = true;
                    }
                }
                else if (e.Text.Equals("/vvscolordemo"))
                {
                    e.Eat = true;
                    redrawCHud();
                }
            }
            catch (Exception ex)
            {
                ErrorLogging.LogError(errorLogFile, ex);
            }
        }

        void Core_ChatBoxMessage(object sender, Decal.Adapter.ChatTextInterceptEventArgs e)
        {
            //TODO: incoming chat handling code
        }
        private void destroyChatEvents()
        {
            // Core.ChatBoxMessage -= new EventHandler<Decal.Adapter.ChatTextInterceptEventArgs>(Core_ChatBoxMessage);
            Core.CommandLineText -= new EventHandler<Decal.Adapter.ChatParserInterceptEventArgs>(Core_CommandLineText);
        }

        private void WriteToChat(string message)
        {
            int target = pluginSettings == null ? 1 : pluginSettings.chatTarget;
            try
            {
                Host.Actions.AddChatText(string.Format("{0}: {1}", PLUGIN, message), MessageColor, target);
            }
            catch (Exception ex)
            {
                ErrorLogging.LogError(errorLogFile, ex);
            }
        }
    }
}